/* ----------------------------------------------------------------------------
  
 Experiment 10:   I2C Kommunikation
 =============    ===============================
  
 Dateiname		: I2C_SimpleSlave.c
  
 Autoren		: Tim Fischer       (Hochschule Heilbronn, Fakultaet TE)
  
 Datum			: 18.11.2023
  
 Version		: 1.0
   
 Hardware		:  Simulide 0.5.16-RC5
  
 Software		:  Entwicklungsumgebung: AtmelStudio 7.0
				   C-Compiler: AVR/GNU C Compiler 5.4.0
   
 Funktion		: TBD
 
 Displayanzeige	: keine
 
 Tastenfunktion	: Die Tastenstellung der Dip-Schalter an Port B werden als Wert ber I2C weitergegeben.
				  Dabei zhlt ein gedrckter Schalter (= hellgrn) als logisches High Signal
 
 Jumperstellung	:  keine
 
 Fuses im uC	: keine
  
   
// ----------------------------------------------------------------------------*/
  
// Deklarationen ==============================================================
  
// Festlegung der Quarzfrequenz
#define F_CPU 16000000UL
#define F_SCL 10000L //100 kHz
 
// Include von Header-Dateien
#include <avr/interrupt.h>
 
// Konstanten
#define SET_BIT(BYTE, BIT)  ((BYTE) |=  (1 << (BIT))) // Bit Zustand in Byte setzen
#define CLR_BIT(BYTE, BIT)  ((BYTE) &= ~(1 << (BIT))) // Bit Zustand in Byte loeschen
#define TGL_BIT(BYTE, BIT)  ((BYTE) ^=  (1 << (BIT))) // Bit Zustand in Byte wechseln (toggle)
 
//Funktionsprototypen
void I2C_Init();
void I2C_setAddress(char Address);
char I2C_readData();
 
uint8_t TWI_Address         =  0b0001010;
uint8_t TWI_AddressMask     = 0b11111110;
 
int main(void)
{
    DDRB= 0xFF;                             // Auf DDRC die Daten ausgeben
    I2C_setAddress(TWI_Address);                // eigene Adresse setzen
     
    while (1) 
    {
        PORTB = I2C_readData();             // Daten an PortC ausgeben
    } 
}
 
//////////////////////////////////////////////////
// Setzen der I2C Adresse auf die der Slave hrt
//////////////////////////////////////////////////
void I2C_setAddress(char Address)                       
{
    TWAR = (Address<<1);                              // Adresse in das Pseudoregister schreiben
    TWAMR= TWI_AddressMask;                             // Adressmaske in das Pseudoregister schreiben
    TWCR = (1<<TWEA)|(1<<TWEN);                         // Enable Ack, Enable Interupt und Enable TIW
 
}
 
//////////////////////////////////////////////////
// Auslesen der bermittelten Daten 
//////////////////////////////////////////////////
char I2C_readData()                     
{
    while (!(TWCR & (1<<TWINT)));                     // warte solange bis TWINT gesetzt ist
    return TWDR;                                        // bermittle Daten
}